@define-color theme_fg_color #000000;
@define-color theme_bg_color #dec9a9;
@define-color theme_text_color #000000;
@define-color theme_base_color #fffbe2;
@define-color theme_view_hover_decoration_color #75511a;
@define-color theme_hovering_selected_bg_color #75511a;
@define-color theme_selected_bg_color #8f6b32;
@define-color theme_selected_fg_color #ffffff;
@define-color theme_view_active_decoration_color #75511a;
@define-color theme_button_background_normal #e2d6b7;
@define-color theme_button_decoration_hover #75511a;
@define-color theme_button_decoration_focus #b3925d;
@define-color theme_button_foreground_normal #000000;
@define-color theme_button_foreground_active #ffffff;
@define-color borders #a7977f;
@define-color warning_color #bfa567;
@define-color success_color #43662e;
@define-color error_color #8b537f;
@define-color theme_unfocused_fg_color #000000;
@define-color theme_unfocused_text_color #000000;
@define-color theme_unfocused_bg_color #dec9a9;
@define-color theme_unfocused_base_color #fffbe2;
@define-color theme_unfocused_selected_bg_color_alt #dbaf6b;
@define-color theme_unfocused_selected_bg_color #dbaf6b;
@define-color theme_unfocused_selected_fg_color #000000;
@define-color theme_button_background_backdrop #e2d6b7;
@define-color theme_button_decoration_hover_backdrop #75511a;
@define-color theme_button_decoration_focus_backdrop #b3925d;
@define-color theme_button_foreground_backdrop #000000;
@define-color theme_button_foreground_active_backdrop #000000;
@define-color unfocused_borders #a7977f;
@define-color warning_color_backdrop #bfa567;
@define-color success_color_backdrop #43662e;
@define-color error_color_backdrop #8b537f;
@define-color insensitive_fg_color #90836e;
@define-color insensitive_base_fg_color #a6a393;
@define-color insensitive_bg_color #dec9a9;
@define-color insensitive_base_color #fffbe2;
@define-color insensitive_selected_bg_color #dec9a9;
@define-color insensitive_selected_fg_color #90836e;
@define-color theme_button_background_insensitive #e2d6b7;
@define-color theme_button_decoration_hover_insensitive #bca880;
@define-color theme_button_decoration_focus_insensitive #d2be98;
@define-color theme_button_foreground_insensitive #938b77;
@define-color theme_button_foreground_active_insensitive #90836e;
@define-color insensitive_borders #cbb89a;
@define-color warning_color_insensitive #e9ddb7;
@define-color success_color_insensitive #bdc7a3;
@define-color error_color_insensitive #d7c0c0;
@define-color insensitive_unfocused_fg_color #90836e;
@define-color theme_unfocused_view_text_color #a6a393;
@define-color insensitive_unfocused_bg_color #dec9a9;
@define-color theme_unfocused_view_bg_color #fffbe2;
@define-color insensitive_unfocused_selected_bg_color #dec9a9;
@define-color insensitive_unfocused_selected_fg_color #90836e;
@define-color theme_button_background_backdrop_insensitive #e2d6b7;
@define-color theme_button_decoration_hover_backdrop_insensitive #bca880;
@define-color theme_button_decoration_focus_backdrop_insensitive #d2be98;
@define-color theme_button_foreground_backdrop_insensitive #938b77;
@define-color theme_button_foreground_active_backdrop_insensitive #90836e;
@define-color unfocused_insensitive_borders #cbb89a;
@define-color warning_color_insensitive_backdrop #e9ddb7;
@define-color success_color_insensitive_backdrop #bdc7a3;
@define-color error_color_insensitive_backdrop #d7c0c0;
@define-color link_color #296fbe;
@define-color link_visited_color #644a9b;
@define-color tooltip_text #25221c;
@define-color tooltip_background #debc85;
@define-color tooltip_border #b0966b;
@define-color content_view_bg #fffbe2;
